/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#include "coloring.h"
#include <basic/macrodefinitions.h>
#include <stdlib.h>

#ifdef WIN32
double rint(double dValue)
{
	return floor(floor(dValue*2+0.00001+1)/2);
}
#endif

#ifndef M_PI
#define M_PI 3.141592654
#endif

unsigned long color_coding::get_rgb(unsigned int nRed, unsigned int nGreen, unsigned int nBlue) const
{
    int ir,ig,ib;
    unsigned long color;

    if ( ColorDepth() == COLOR_DEPTH_24 )
    {
	ir = (nRed&0xff)<<16;
	ig = (nGreen&0xff)<<8;
	ib = (nBlue&0xff);
    }
    else
    {
	ir = (nRed&0x1f)<<11;
	ig = (nGreen&0x1f)<<6;
	ib = (nBlue&0x3f);	
    }
  
    color = ir|ig|ib;
  
    return color;  
}

colored_object::colored_object(unsigned short r, unsigned short g, unsigned short b)
{
    red = r;
    green = g;
    blue = b;
}

void colored_object::set_color_rgb(unsigned short r, unsigned short g, unsigned short b)
{
    red = r;
    green = g;
    blue = b;
}

unsigned long colored_object::get_color_X11(double f)
{
    unsigned long r, g, b, res;
    unsigned short rp,gp,bp;

    assert(f>=-1.&&f<=1.);

    if(f>=0.){
	rp = (unsigned short)(f*(double)(255-red));
	gp = (unsigned short)(f*(double)(255-green));
	bp = (unsigned short)(f*(double)(255-blue));

	r = ((unsigned long)(red+rp))*32/256;
	g = ((unsigned long)(green+gp))*32/256;
	b = ((unsigned long)(blue+bp))*32/256;
    }
    else{
	rp = (unsigned short)((1.+f)*(double)(red));
	gp = (unsigned short)((1.+f)*(double)(green));
	bp = (unsigned short)((1.+f)*(double)(blue));

	r = ((unsigned long)(rp))*32/256;
	g = ((unsigned long)(gp))*32/256;
	b = ((unsigned long)(bp))*32/256;
    }

    r<<=11;
    g<<=6;
    res = r;
    res |= g;
    res |= b;
	
    return res;
}

unsigned long color_coding::get_rgb(double r, double g, double b) const
{
    ASSERT(r<=1. && r >=0.);
    ASSERT(g<=1. && g >=0.);
    ASSERT(b<=1. && b >=0.);

    int ir,ig,ib;
    unsigned long color;

    if (ColorDepth() == COLOR_DEPTH_24)
    {
	ir = ((int)rint(r*((double)0xff)))<<16;
	ig = ((int)rint(g*((double)0xff)))<<8;
	ib = (int)rint(b*((double)0xff));
    }
    else
    {
	ir = ((int)rint(r*((double)0x1f)))<<11;
	ig = ((int)rint(g*((double)0x1f)))<<6;
	ib = (int)rint(b*((double)0x3f));	
    }
	
    color = ir|ig|ib;

    return color;
}

unsigned long color_coding::get_cmy(double c, double m, double y)
{
    int ic,im,iy;
    unsigned long color;

    ic = ((int)rint((1.-c)*((double)0x1f)))<<11;
    im = ((int)rint((1.-m)*((double)0x1f)))<<6;
    iy = (int)rint((1.-y)*((double)0x3f));

    color = ic|im|iy;

    return color;
}

unsigned long color_coding::get_hsv(double h, double s, double v)
{
    ASSERT(0. <= h && h <= M_PI);
    ASSERT(0. <= s && s <= 1.);
    ASSERT(0. <= v && v <= 1.);

    int r,g,b;
    unsigned long color;

    if(s==0.){   // The color is on the black-and-white center line
	// Achromatic color: there is no hue
	if(h<0.){
	    if ( ColorDepth() == COLOR_DEPTH_24 )
	    {
      		r = g = b = (int)rint(v*((double)0xff));
	    }
	    else
	    {
      		r  = (int)rint(v*((double)0x1f));
      		g  = (int)rint(v*((double)0x1f));
      		b  = (int)rint(v*((double)0x3f));      				
	    }
	}
	else{
	    printf("Error: impossible color (%f,%f,%f)\n",h,s,v);
	    exit(1);
	}
    }
    else{
	double f,p,q,t;
	double i;

	if(h==2.*M_PI)
	    h = 0.;

	// Convert from radians to degree!

	h = h/M_PI*180.;

	h /= 60.;
    
	i = floor(h);
	f = h - i;
	p = v*(1.-s);
	q = v*(1.-(s*f));
	t = v*(1.-(s*(1.-f)));
    
	double faktor1, faktor2;
    
	if ( ColorDepth() == COLOR_DEPTH_24)
	{
	    faktor1 = (double)0xff;
	    faktor2 = (double)0xff;
	}
	else
	{
	    faktor1 = (double)0x1f;
	    faktor2 = (double)0x3f;
	}
    
	switch(((int)i)){
	    case 0:
		r = (int)rint(v*faktor1);
		g = (int)rint(t*faktor1);
		b = (int)rint(p*faktor2);
		break;
	    case 1:
		r = (int)rint(q*faktor1);
		g = (int)rint(v*faktor1);
		b = (int)rint(p*faktor2);
		break;
	    case 2:
		r = (int)rint(p*faktor1);
		g = (int)rint(v*faktor1);
		b = (int)rint(t*faktor2);
		break;
	    case 3:
		r = (int)rint(p*faktor1);
		g	 = (int)rint(q*faktor1);
		b = (int)rint(v*faktor2);
		break;
	    case 4:
		r = (int)rint(t*faktor1);
		g = (int)rint(p*faktor1);
		b = (int)rint(v*faktor2);
		break;
	    case 5:
		r = (int)rint(v*faktor1);
		g = (int)rint(p*faktor1);
		b = (int)rint(q*faktor2);
		break; 
	}
    }
  
    if ( ColorDepth() == COLOR_DEPTH_24 )
    {		
  	r = r<<16;
  	g = g<<8;
    }
    else
    {
  	r = r<<11;
  	g = g<<6;	
    }

    color = r|g|b;

    return color;
}

double color_coding::get_red(unsigned long color)
{
    if ( ColorDepth() == COLOR_DEPTH_24 )
    {
	return(((double)((color&0xff0000)>>16))/((double)0xff));
    }
    else
    {
	return(((double)((color&0xf8000)>>11))/((double)0x1f));
    }
}

double color_coding::get_green(unsigned long color)
{
    if ( ColorDepth() == COLOR_DEPTH_24 )
    {
	return(((double)((color&0xff00)>>8))/((double)0xff));
    }
    else
    {
	return(((double)((color&0x7c0)>>6))/((double)0x1f));	
    }
}

double color_coding::get_blue(unsigned long color)
{
    if ( ColorDepth() == COLOR_DEPTH_24 )
    {
	return(((double)(color&0xff))/((double)0xff));
    }
    else
    {
	return(((double)(color&0x3f))/((double)0x3f));  	
    }
}

double color_coding::get_cyan(unsigned long color)
{
    if ( ColorDepth() == COLOR_DEPTH_24 )
    {
	return(((double)(0xff-((color&0xff0000)>>16)))/((double)0xff));
    }
    else
    {
	return(((double)(0x1f-((color&0xf8000)>>11)))/((double)0x1f));	
    }
}

double color_coding::get_magenta(unsigned long color)
{
    // Doesn't work!
    assert(false);
    return(((double)(0x1f-((color&0x7c0)>>6)))/((double)0x1f));
}

double color_coding::get_yellow(unsigned long color)
{
    // Doesn't work!
    assert(false);
    return(((double)(0x3f-(color&0x3f)))/((double)(0x3f)));
}

double color_coding::get_hue(unsigned long color)
{
    int r,g,b,max,min;
    double delta;
    double dr,dg,db;
    double hue;

    if(color==0x000000) // Undefined case
    {
	return -1.;
    }
    else{
  	if ( ColorDepth() == COLOR_DEPTH_24 )
  	{
	    r = (color&0xff0000)>>16;
	    g = (color&0xff00)>>8;
	    b = color&0xff;
	}
	else
	{
	    r = (color&0xf8000)>>11;
	    g = (color&0x7c0)>>6;
	    b = color&0x3f;	
	}
	
	min = minimum(r,g,b);
	max = maximum(r,g,b);

	if ( ColorDepth() == COLOR_DEPTH_24 )
	{
	    dr = ((double)r)/((double)0xff);
	    dg = ((double)g)/((double)0xff);
	    db = ((double)b)/((double)0xff);
	}
	else
	{
	    dr = ((double)r)/((double)0x1f);
	    dg = ((double)g)/((double)0x1f);
	    db = ((double)b)/((double)0x3f);	
	}
	
	if ( ColorDepth() == COLOR_DEPTH_24 )
	{
	    delta = ((double)(max-min))/((double)0xff);
	}
	else
	{
	    // This case seems a little tricky... maybe it's wrong!
		
	    delta = ((double)(max-min))/5.5;		
	}
	
	if(r==max)
	    hue = (dg-db)/delta;
	else if(g==max)
	    hue = 2.0 + (db-dr)/delta;
	else if(b==max)
	    hue = 4.0 + (dr-dg)/delta;
	hue *= 60.;
	if(hue<0.)
	    hue += 360.;

	hue = hue/180.*M_PI;

	return hue;
    }
}

double color_coding::get_saturation(unsigned long color)
{
    int r,g,b,max,min;
    double saturation, dmax, dmin;

    if ( ColorDepth() == COLOR_DEPTH_24 )
    {
	r = (color&0xff0000)>>16;
	g = (color&0xff00)>>8;
	b = color&0xff;
    }
    else
    {
	r = (color&0xf8000)>>11;
	g = (color&0x7c0)>>6;
	b = color&0x3f;  		
    }
  	
    min = minimum(r,g,b);
    max = maximum(r,g,b);

    dmin = ((double)min)/((double)0xff);
    dmax = ((double)max)/((double)0xff);

    saturation = (max!=0)?((dmax-dmin)/dmax):0.;

    return saturation;
}

double color_coding::get_value(unsigned long color)
{
    int r,g,b,max;
    double dmax;

    if ( ColorDepth() == COLOR_DEPTH_24 )
    {
	r = (color&0xff0000)>>16;
	g = (color&0xff00)>>8;
	b = color&0xff;
    }
    else
    {
	r = (color&0xf8000)>>11;
	g = (color&0x7c0)>>6;
	b = color&0x3f;  	
    }
  	
    max = maximum(r,g,b);

    if ( ColorDepth() == COLOR_DEPTH_24 )
    {
	dmax = ((double)max)/((double)0xff);
    }
    else
    {
	// This case seems a little tricky... maybe it's wrong!
		
	dmax = ((double)max)/5.5;		
    }
	
    return dmax;
}

int color_coding::maximum(int r, int g, int b)
{
    if(r>=g){
	if(r>=b){
	    return r;
	}
	else{
	    return b;
	}
    }
    else if(g>=b){
	return g;
    }
    else{
	return b;
    }
}

int color_coding::minimum(int r, int g, int b)
{
    if(r<=g)
    {
    	if(r<=b)
    	{
	    return r;
    	}
    	else
    	{
	    return b;
    	}
    }
    else if(g<=b)
    {
    	return g;
    }
    else
    {
    	return b;
    }
}

#ifndef WIN32
color_coding::color_coding(Display *disp)
#else
color_coding::color_coding()
#endif
{
#ifndef WIN32
    if ( disp == NULL )
    {
#endif
	SetColorDepth(COLOR_DEPTH_24);
#ifndef WIN32
	}
    else
    {
	int screen = DefaultScreen(disp);
	int depth = DefaultDepth(disp,screen);
		
	if ( depth == 16 )
	{
	    SetColorDepth(COLOR_DEPTH_16);
	}
	else
	{
	    SetColorDepth(COLOR_DEPTH_24);
	}
    }
#endif
}










