/*******************************************************************************
 *
 * libgui Version 0.0.1
 *
 * libgui is a simple framework of GUI classes for X11/Motif widgets written in C++.
 *
 * The full text of the GNU General Public License can be found in the doc 
 * directory.
 *
 * Copyright (C) 2002, 2003 Stefan Hoefer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You can contact Stefan Hoefer via email at stefan@hoefer.ch
 *
 ******************************************************************************/

#ifndef COLORING_H

#define COLORING_H

#include "assert.h"
#include <math.h>
#include <stdio.h>
#include <X11/Intrinsic.h>

/*! \class colored_object
  \brief A class that handles colors of objects.
*/

class colored_object
{
 private: 
    unsigned short red;   /*< holds the red value of the color of the object */
    unsigned short green; /*< holds the green value of the color of the object */
    unsigned short blue;  /*< holds the blue value of the color of the object */
 public:
/*! 
  \fn unsigned long get_rgb(unsigned int nRed, unsigned int nGreen, unsigned int nBlue) const
  \brief Determines the 24-Bit value of the color specified by the parameters.
  \param nRed The red value of the desired color.
  \param nGreen The green value of the desired color.
  \param nBlue The blue value of the desired color.
  \return Returns a 24-Bit encoded value representing the desired color.
*/
    colored_object(unsigned short r, unsigned short g, unsigned short b);
/*! 
  \fn double get_saturation(unsigned long color)
  \brief Extracts the saturation value from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The saturation value of the color, in the range [0.,1.].
*/
    void set_color_rgb(unsigned short r, unsigned short g, unsigned short b);
/*! 
  \fn unsigned long get_color_X11(double f)
  \brief Returns the current color of the object in 24bit (8+8+8) representation.
  \param f allows the dimming of the color (i.e. modifies the brightness)
 */
    unsigned long get_color_X11(double f);
};

/*! \class color_coding
  \brief A class that supports color representation conversions.
*/

class color_coding
{
 public:
    enum COLOR_DEPTH {COLOR_DEPTH_16,COLOR_DEPTH_24}; /*< Currently, color depths of 16bit and 24bit are supported */
 private:
    COLOR_DEPTH m_nDepth; /*< The depth that is currently assumed when converting color values */
		
    // Colors are internally coded as 24-bit RGB!
 private:
    int minimum(int,int,int);
    int maximum(int,int,int);
 public:
    // RGB
  
/*! 
  \fn double get_red(unsigned long color)
  \brief Extracts the red value from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The red value of the color, in the range [0.,1.]
 */
    double get_red(unsigned long);
/*! 
  \fn double get_green(unsigned long color)
  \brief Extracts the green value from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The green value of the color, in the range [0.,1.]
 */
    double get_green(unsigned long);
/*! 
  \fn double get_blue(unsigned long color)
  \brief Extracts the blue value from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The blue value of the color, in the range [0.,1.]
 */
    double get_blue(unsigned long);
/*! 
  \fn unsigned long get_rgb(double r, double g, double b)
  \brief Returns the 24- or 16bit RGB value of the color specified by the parameters
  \param r The red value of the desired color, in the range [0,1]
  \param g The green value of the desired color, in the range [0,1]
  \param b The blue value of the desired color, in the range [0,1]
  \return The 16- (5+6+5) or 24bit (8+8+8) representation of the color
*/
    unsigned long get_rgb(double, double, double) const;
    unsigned long get_rgb(unsigned int nRed, unsigned int nGreen, unsigned int nBlue) const;

    // CYM

/*! 
  \fn double get_cyan(unsigned long color)
  \brief Extracts the cyan value from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The cyan value of the color, in the range [0.,1.]
 */
    double get_cyan(unsigned long);
/*! 
  \fn double get_yellow(unsigned long color)
  \brief Extracts the yellow value from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The yellow value of the color, in the range [0.,1.]
 */
    double get_yellow(unsigned long);
/*! 
  \fn double get_magenta(unsigned long color)
  \brief Extracts the magenta value from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The magenta value of the color, in the range [0.,1.]
 */
    double get_magenta(unsigned long);
/*! 
  \fn unsigned long get_cmy(double r, double g, double b)
  \brief Returns the 24- or 16bit CMY value of the color specified by the parameters
  \param c The cyan value of the desired color, in the range [0,1]
  \param m The magenta value of the desired color, in the range [0,1]
  \param y The yellow value of the desired color, in the range [0,1]
  \return The 16- (5+6+5) or 24bit (8+8+8) representation of the color
*/
    unsigned long get_cmy(double, double, double);

    // HSV

/*! 
  \fn double get_hue(unsigned long color)
  \brief Extracts the hue value from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The hue value of the color, in the range [0.,1.], or, if the color is black, -1.
 */
    double get_hue(unsigned long);
/*! 
  \fn double get_saturation(unsigned long color)
  \brief Extracts the saturation value from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The saturation value of the color, in the range [0.,1.].
 */
    double get_saturation(unsigned long);
/*! 
  \fn double get_value(unsigned long color)
  \brief Extracts the value (brightness) from a 24bit or 16bit RGB color representation
  \param color The 24bit or 16bit color representation
  \return The brightness of the color, in the range [0.,1.].
 */
    double get_value(unsigned long);
/*! 
  \fn unsigned long get_hsv(double h, double s, double v)
  \brief Returns the 24- or 16bit HSV value of the color specified by the parameters
  \param h The hue value of the color, 0 <= h <= PI
  \param s The saturation value of the color, 0 <= s <= 1
  \param v The value of the color (brightness), 0 <= v <= 1
  \return The 16- (5+6+5) or 24bit (8+8+8) representation of the color
*/  
    unsigned long get_hsv(double, double, double);
  
#ifndef WIN32
	color_coding(Display *disp=NULL);
#else
	color_coding();
#endif
	inline void SetColorDepth(COLOR_DEPTH nDepth) { m_nDepth = nDepth;}
    inline COLOR_DEPTH ColorDepth() const { return m_nDepth; }
};

#endif






